@@ -61,6 +61,7 @@ static int associate_schema_mode = FALSE;
 static int dissociate_schema_mode = FALSE;
 static int default_source_mode = FALSE;
 static int recursive_unset_mode = FALSE;
+static int do_version = FALSE;
 
 struct poptOption options[] = {
   { 
@@ -352,6 +353,15 @@ struct poptOption options[] = {
     N_("Get the name of the default source"),
     NULL
   },
+  {
+    "version",
+    'v',
+    POPT_ARG_NONE,
+    &do_version,
+    0,
+    N_("Print version"),
+    NULL
+  },
   {
     NULL,
     '\0',
@@ -398,6 +408,8 @@ main (int argc, char** argv)
   xmlKeepBlanksDefault(1);
 
   setlocale (LC_ALL, "");
+  _gconf_init_i18n ();
+  textdomain (GETTEXT_PACKAGE);
   
   ctx = poptGetContext("gconftool", argc, (const char **) argv, options, 0);
 
@@ -417,6 +429,12 @@ main (int argc, char** argv)
 
   /* Um, this is a mess. Not using popt right? */
 
+  if (do_version)
+    {
+      g_print ("%s\n", VERSION);
+      return 0;
+    }
+  
   if ((get_mode && set_mode) ||
       (get_mode && unset_mode))
     {
@@ -943,10 +961,10 @@ list_pairs_in_dir(GConfEngine* conf, const gchar* dir, guint depth)
           GConfEntry* pair = tmp->data;
           gchar* s;
 
-          if (pair->value)
-                  s = gconf_value_to_string(pair->value);
+          if (gconf_entry_get_value (pair))
+            s = gconf_value_to_string (gconf_entry_get_value (pair));
           else
-                  s = g_strdup(_("(no value set)"));
+            s = g_strdup(_("(no value set)"));
           
           printf(" %s%s = %s\n", whitespace,
                  gconf_key_key (gconf_entry_get_key (pair)),
@@ -1992,6 +2010,25 @@ extract_global_info(xmlNodePtr node,
     }
 
   g_assert(info->key != NULL);
+
+  if (info->type == GCONF_VALUE_LIST &&
+      info->list_type == GCONF_VALUE_INVALID)
+    {
+      fprintf(stderr, _("WARNING: no <list_type> specified for schema of type list\n"));
+    }
+
+
+  if (info->type == GCONF_VALUE_PAIR &&
+      info->car_type == GCONF_VALUE_INVALID)
+    {
+      fprintf(stderr, _("WARNING: no <car_type> specified for schema of type pair\n"));
+    }
+
+  if (info->type == GCONF_VALUE_PAIR &&
+      info->cdr_type == GCONF_VALUE_INVALID)
+    {
+      fprintf(stderr, _("WARNING: no <cdr_type> specified for schema of type pair\n"));
+    }
   
   /* Have to do this last, because the type may come after the default
      value
